@@ -108,6 +108,43 @@ list_double_clicked_cb (GnomeDbGrid *grid, gint row, gpointer user_data)
 	dsn_config_edit_properties (GTK_WIDGET (user_data));
 }
 
+static void
+list_popup_properties_cb (GtkWidget *menu, gpointer user_data)
+{
+	dsn_config_edit_properties (GTK_WIDGET (user_data));
+}
+
+static void
+list_popup_delete_cb (GtkWidget *menu, gpointer user_data)
+{
+	dsn_config_delete (GTK_WIDGET (user_data));
+}
+
+static void
+list_popup_cb (GnomeDbGrid *grid, GtkMenu *menu, gpointer user_data)
+{
+	GtkWidget *item_delete, *item_properties;
+	gboolean ok;
+
+	item_delete = gnome_db_new_menu_item (GTK_STOCK_DELETE,
+					      TRUE,
+					      G_CALLBACK (list_popup_delete_cb),
+					      user_data);
+	item_properties = gnome_db_new_menu_item (GTK_STOCK_PROPERTIES,
+						  TRUE,
+						  G_CALLBACK (list_popup_properties_cb),
+						  user_data);
+
+	ok = gnome_db_grid_get_selection (grid) != NULL;
+	gtk_widget_set_sensitive (item_delete, ok);
+	gtk_widget_set_sensitive (item_properties, ok);
+
+	gtk_menu_prepend (menu, gtk_separator_menu_item_new ());
+	gtk_menu_prepend (menu, item_delete);
+	gtk_menu_prepend (menu, gtk_separator_menu_item_new ());
+	gtk_menu_prepend (menu, item_properties);
+}
+
 /*
  * Public functions
  */
@@ -152,6 +189,8 @@ dsn_config_new (void)
 	gtk_widget_show (priv->dsn_list);
 	g_signal_connect (G_OBJECT (priv->dsn_list), "double_clicked",
 			  G_CALLBACK (list_double_clicked_cb), dsn);
+	g_signal_connect (G_OBJECT (priv->dsn_list), "create_popup_menu",
+			  G_CALLBACK (list_popup_cb), dsn);
 	gtk_table_attach (GTK_TABLE (table), priv->dsn_list, 0, 1, 0, 1,
 			  GTK_FILL | GTK_SHRINK | GTK_EXPAND,
 			  GTK_FILL | GTK_SHRINK | GTK_EXPAND,
@@ -217,26 +256,52 @@ dsn_config_edit_properties (GtkWidget *dsn)
 static void
 foreach_remove_cb (GnomeDbGrid *grid, gint row, gpointer user_data)
 {
+	DsnConfigPrivate *priv;
 	GdaDataModel *model;
+	GtkWidget *dialog;
+	GtkWidget *button;
 	gchar *str;
-	DsnConfigPrivate *priv = (DsnConfigPrivate *) user_data;
+	gchar *dsn;
 
+	priv = (DsnConfigPrivate *) user_data;
 	model = gnome_db_grid_get_model (GNOME_DB_GRID (priv->dsn_list));
 	if (!GDA_IS_DATA_MODEL (model))
 		return;
 
-	str = gda_value_stringify (
+	dsn = gda_value_stringify (
 		(GdaValue *) gda_data_model_get_value_at (model, 0, row));
-	if (!str)
+	if (dsn == NULL)
 		return;
 
-	if (gnome_db_yes_no_dialog (
-		    gtk_widget_get_toplevel (priv->dsn_list),
-		    _("Do you want to remove '%s' data source?"),
-		    str)) {
-		gnome_db_config_remove_data_source (str);
-	}
+	str = g_strdup_printf (
+		_("Are you sure you want to remove the data source '%s'?"),
+		dsn);
 
+	dialog = gnome_db_new_alert (
+		GTK_WINDOW (gtk_widget_get_toplevel (priv->dsn_list)),
+		GTK_MESSAGE_QUESTION,
+		str,
+		NULL);
+		
+	button = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
+	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+	gtk_dialog_add_action_widget (GTK_DIALOG (dialog),
+				      button,
+				      GTK_RESPONSE_CANCEL);
+	button = gtk_button_new_from_stock (GTK_STOCK_DELETE);
+	gtk_dialog_add_action_widget (GTK_DIALOG (dialog),
+				      button,
+				      GTK_RESPONSE_OK);
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog),
+					 GTK_RESPONSE_CANCEL);
+	
+	gtk_widget_show_all (dialog);
+	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK) 
+		gnome_db_config_remove_data_source (dsn);
+
+	/* free memory */
+	gtk_widget_destroy (dialog);
+	g_free (dsn);
 	g_free (str);
 }
 
@@ -250,3 +315,30 @@ dsn_config_delete (GtkWidget *dsn)
 					(GnomeDbGridForeachFunc) foreach_remove_cb,
 					priv);
 }
+
+void
+dsn_config_select_all (GtkWidget *dsn)
+{
+	DsnConfigPrivate *priv;
+
+	priv = g_object_get_data (G_OBJECT (dsn), DSN_CONFIG_DATA);
+	gnome_db_grid_select_all (GNOME_DB_GRID (priv->dsn_list));	
+}
+
+void
+dsn_config_unselect_all (GtkWidget *dsn)
+{
+	DsnConfigPrivate *priv;
+
+	priv = g_object_get_data (G_OBJECT (dsn), DSN_CONFIG_DATA);
+	gnome_db_grid_unselect_all (GNOME_DB_GRID (priv->dsn_list));	
+}
+
+gboolean
+dsn_config_has_selection (GtkWidget *dsn)
+{
+	DsnConfigPrivate *priv;
+
+	priv = g_object_get_data (G_OBJECT (dsn), DSN_CONFIG_DATA);
+	return gnome_db_grid_get_selection (GNOME_DB_GRID (priv->dsn_list)) != NULL;
+}
