@@ -789,9 +789,9 @@ gnome_db_grid_set_title (GnomeDbGrid *grid, const gchar *title)
 
 /**
  * gnome_db_grid_set_column_title
- * @grid a #GnomeDbGrid widget
- * @col the number of the column to change the title
- * @title the new title for this column
+ * @grid: a #GnomeDbGrid widget
+ * @col: the number of the column to change the title
+ * @title: the new title for this column
  *
  * Sets the title for a column
  */
@@ -813,8 +813,8 @@ gnome_db_grid_set_column_title (GnomeDbGrid *grid, gint col, const gchar *title)
 
 /** 
  * gnome_db_grid_get_column_title
- * @grid a #GnomeDbGrid widget
- * @col the number of the column to get the title
+ * @grid: a #GnomeDbGrid widget
+ * @col: the number of the column to get the title
  *
  * Get the title of a column
  * Returns: the title of a column.
@@ -1060,6 +1060,29 @@ gnome_db_grid_foreach_selected (GnomeDbGrid *grid,
 		foreach_func (grid, GPOINTER_TO_INT (l->data), user_data);
 }
 
+/**
+ * gnome_db_grid_select_row:
+ * @grid: a #GnomeDbGrid widget.
+ * @row: the number of the row to be selected.
+ *
+ * Select the given row in the #GnomeDbGrid widget.
+ */
+void
+gnome_db_grid_select_row (GnomeDbGrid *grid, gint row)
+{
+	GtkTreePath *path;
+	
+	g_return_if_fail (GNOME_DB_IS_GRID (grid));
+	
+	path = gtk_tree_path_new ();
+	gtk_tree_path_append_index (path, row);
+	
+	gtk_tree_selection_select_path (gtk_tree_view_get_selection (GTK_TREE_VIEW (grid->priv->tree_view)),
+					path);
+
+	gtk_tree_path_free (path);
+}
+
 /**
  * gnome_db_grid_select_all
  * @grid: a #GnomeDbGrid widget.
