@@ -26,6 +26,7 @@
 #include <glib-object.h>
 #include <gtk/gtkbutton.h>
 #include <gtk/gtkcheckbutton.h>
+#include <gtk/gtkcheckmenuitem.h>
 #include <gtk/gtkcombo.h>
 #include <gtk/gtkfilesel.h>
 #include <gtk/gtkframe.h>
@@ -33,15 +34,18 @@
 #include <gtk/gtkhbox.h>
 #include <gtk/gtkhpaned.h>
 #include <gtk/gtkhseparator.h>
+#include <gtk/gtkimage.h>
+#include <gtk/gtkimagemenuitem.h>
 #include <gtk/gtklabel.h>
-#include <gtk/gtkmenu.h>
 #include <gtk/gtkmain.h>
+#include <gtk/gtkmenu.h>
 #include <gtk/gtkmenuitem.h>
 #include <gtk/gtkmenushell.h>
 #include <gtk/gtkmessagedialog.h>
 #include <gtk/gtknotebook.h>
 #include <gtk/gtkradiobutton.h>
 #include <gtk/gtkscrolledwindow.h>
+#include <gtk/gtkseparatormenuitem.h>
 #include <gtk/gtkstatusbar.h>
 #include <gtk/gtkstock.h>
 #include <gtk/gtktable.h>
@@ -267,7 +271,7 @@ gnome_db_new_label_widget (const gchar *text)
 {
 	GtkWidget *label;
 
-	label = gtk_label_new (text);
+	label = gtk_label_new_with_mnemonic (text);
 	gtk_label_set_selectable (GTK_LABEL (label), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	gtk_widget_show (label);
@@ -630,6 +634,8 @@ gnome_db_option_menu_set_selection (GtkOptionMenu *option_menu, const gchar *sel
 	g_return_if_fail (selection != NULL);
 
 	menu = gtk_option_menu_get_menu (option_menu);
+	if (!GTK_IS_MENU_SHELL (menu))
+		return;
 
 	for (l = GTK_MENU_SHELL (menu)->children, i = 0; l != NULL; l = l->next, i++) {
 		const gchar *str;
@@ -800,39 +806,83 @@ gnome_db_select_file_dialog (GtkWidget *parent, const gchar *title)
 }
 
 /**
- * gnome_db_yes_no_dialog
+ * gnome_db_new_alert
  */
-gboolean
-gnome_db_yes_no_dialog (GtkWidget *parent, const gchar *format, ...)
+GtkWidget *
+gnome_db_new_alert (GtkWindow *parent,
+		    GtkMessageType type,
+		    const gchar *primary_text,
+		    const gchar *secondary_text)
 {
-	va_list args;
-        gchar sz[2048];
+	const gchar *stock_id = NULL;
+	gchar *str;
 	GtkWidget *dialog;
-	GdkPixbuf *icon;
-	gboolean retval = FALSE;
+	GtkWidget *image;
+	GtkWidget *label;
+	GtkWidget *hbox;
+	GtkWidget *vbox;
 
-	/* build the message string */
-        va_start (args, format);
-        vsprintf (sz, format, args);
-        va_end (args);
+	g_return_val_if_fail (primary_text != NULL, NULL);
 
-	/* create the dialog */
-	dialog = gtk_message_dialog_new (GTK_WINDOW (parent),
-					 GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION,
-					 GTK_BUTTONS_YES_NO, sz);
-	gtk_window_set_title (GTK_WINDOW (dialog), _("Question"));
+	dialog = gtk_dialog_new_with_buttons ("",
+					     parent,
+					     GTK_DIALOG_DESTROY_WITH_PARENT |
+					     GTK_DIALOG_NO_SEPARATOR |
+					     GTK_DIALOG_MODAL,
+					     NULL);
+					     
+	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+	gtk_container_set_border_width (GTK_CONTAINER (dialog), 6);
+	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 12);
 
-	icon = gdk_pixbuf_new_from_file (LIBGNOMEDB_ICONSDIR "/gnome-db.png", NULL);
-	if (icon) {
-		gtk_window_set_icon (GTK_WINDOW (dialog), icon);
-		g_object_unref (icon);
+	hbox = gtk_hbox_new (FALSE, 12);
+	gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox,
+			    TRUE, TRUE, 0);
+
+	switch (type) {
+	case GTK_MESSAGE_INFO:
+		stock_id = GTK_STOCK_DIALOG_INFO;
+		break;
+	case GTK_MESSAGE_WARNING:
+		stock_id = GTK_STOCK_DIALOG_WARNING;
+		break;
+	case GTK_MESSAGE_QUESTION:
+		stock_id = GTK_STOCK_DIALOG_QUESTION;
+		break;
+	case GTK_MESSAGE_ERROR:
+		stock_id = GTK_STOCK_DIALOG_ERROR;
+		break;
+	}
+	g_return_val_if_fail (stock_id != NULL, NULL);
+
+	image = gtk_image_new_from_stock (stock_id,
+					  GTK_ICON_SIZE_DIALOG);
+	gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0.0);
+	gtk_box_pack_start (GTK_BOX (hbox), image, TRUE, TRUE, 0);
+
+	vbox = gtk_vbox_new (FALSE, 6);
+	gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
+
+	label = gtk_label_new (NULL);
+	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+	str = g_strconcat ("<span weight=\"bold\" size=\"larger\">",
+			   primary_text,
+			   "</span>",
+			   NULL);
+	gtk_label_set_markup (GTK_LABEL (label), str);
+	g_free (str);
+	gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
+
+	if (secondary_text != NULL) {
+		label = gtk_label_new (secondary_text);
+		gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+		gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+		gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
 	}
 
-	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES)
-		retval = TRUE;
-	gtk_widget_destroy (dialog);
-
-	return retval;
+	return dialog;
 }
 
 /**
@@ -847,12 +897,15 @@ gnome_db_show_error (const gchar *format, ...)
 
 	/* build the message string */
         va_start (args, format);
-        vsprintf (sz, format, args);
+        vsnprintf (sz, sizeof sz, format, args);
         va_end (args);
 
 	/* create the error message dialog */
-	dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR,
-					 GTK_BUTTONS_CLOSE, sz);
+	dialog = gnome_db_new_alert (NULL, GTK_MESSAGE_ERROR, sz, NULL);
+	gtk_dialog_add_action_widget (GTK_DIALOG (dialog),
+				      gtk_button_new_from_stock (GTK_STOCK_OK),
+				      GTK_RESPONSE_OK);
+	gtk_widget_show_all (dialog);
 	gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
 }
@@ -872,12 +925,15 @@ gnome_db_show_message (const gchar *format, ...)
 
 	/* build the message string */
         va_start (args, format);
-        vsprintf (sz, format, args);
+        vsnprintf (sz, sizeof sz, format, args);
         va_end (args);
 
-	/* create the error message dialog */
-	dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_INFO,
-					 GTK_BUTTONS_CLOSE, sz);
+	/* create the info message dialog */
+	dialog = gnome_db_new_alert (NULL, GTK_MESSAGE_INFO, sz, NULL);
+	gtk_dialog_add_action_widget (GTK_DIALOG (dialog),
+				      gtk_button_new_from_stock (GTK_STOCK_OK),
+				      GTK_RESPONSE_OK);
+	gtk_widget_show_all (dialog);
 	gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
 }
@@ -918,3 +974,39 @@ gnome_db_pop_cursor_busy (GtkWidget *window)
                 g_object_set_data (G_OBJECT (window), "Panel:WindowBusy", GINT_TO_POINTER (busy));
         }
 }
+
+GtkWidget *
+gnome_db_new_check_menu_item (const gchar *label,
+			      gboolean active,
+			      GCallback cb_func,
+			      gpointer user_data)
+{
+	GtkWidget *item;
+	
+	item = gtk_check_menu_item_new_with_mnemonic (label);
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item), active);
+	
+	g_signal_connect (G_OBJECT (item), "toggled",
+			  G_CALLBACK (cb_func), user_data);
+
+	return item;
+}
+
+GtkWidget *
+gnome_db_new_menu_item (const gchar *label,
+		 	gboolean pixmap,
+		        GCallback cb_func,
+			gpointer user_data)
+{
+	GtkWidget *item;
+
+	if (pixmap)
+		item = gtk_image_menu_item_new_from_stock (label, NULL);
+	else
+		item = gtk_menu_item_new_with_mnemonic (label);
+	
+	g_signal_connect (G_OBJECT (item), "activate",
+			  G_CALLBACK (cb_func), user_data);
+
+	return item;
+}
